@@ -1,6 +1,7 @@
 # Orca
 #
-# Copyright 2005-2009 Sun Microsystems Inc.
+# Copyright 2009-2010 Sun Microsystems Inc.
+# Copyright 2010 Joanmarie Diggs
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -22,14 +23,14 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
+__copyright__ = "Copyright (c) 2009-2010 Sun Microsystems Inc."  \
+                "Copyright (c) 2010 Joanmarie Diggs"
 __license__   = "LGPL"
 
+import gtk
 import pyatspi
 
-import orca.braille as braille
 import orca.default as default
-import orca.input_event as input_event
 import orca.orca as orca
 import orca.orca_state as orca_state
 import orca.settings as settings
@@ -40,6 +41,9 @@ from orca.orca_i18n import _
 from braille_generator import BrailleGenerator
 from speech_generator import SpeechGenerator
 from tutorialgenerator import TutorialGenerator
+from script_utilities import Utilities
+
+import script_settings
 
 ########################################################################
 #                                                                      #
@@ -59,6 +63,11 @@ class Script(default.Script):
         default.Script.__init__(self, app)
         self._isBusy = False
         self._lastObjectPresented = None
+        self._presentedStatusBarIcon = False
+
+        # Initialize variable to None to make pylint happy.
+        #
+        self.presentLoggedErrorsCheckButton = None
 
     def getListeners(self):
         """Sets up the AT-SPI event listeners for this script."""
@@ -83,6 +92,53 @@ class Script(default.Script):
 
         return TutorialGenerator(self)
 
+    def getUtilities(self):
+        """Returns the utilites for this script."""
+
+        return Utilities(self)
+
+    def getAppPreferencesGUI(self):
+        """Return a GtkVBox contain the application unique configuration
+        GUI items for the current application.
+        """
+
+        vbox = gtk.VBox(False, 0)
+        vbox.set_border_width(12)
+        gtk.Widget.show(vbox)
+
+        # Translators: The Package Manager application notifies the
+        # user of minor errors by displaying an icon in the status
+        # bar and adding them to an error log rather than displaying
+        # the error in a dialog box. This string is the label for a
+        # checkbox. If it is checked, Orca will inform the user when
+        # the notification icon has appeared.
+        #
+        label = _("Notify me when errors have been logged.")
+        self.presentLoggedErrorsCheckButton = gtk.CheckButton(label)
+        gtk.Widget.show(self.presentLoggedErrorsCheckButton)
+        gtk.Box.pack_start(vbox, self.presentLoggedErrorsCheckButton,
+                           False, False, 0)
+        gtk.ToggleButton.set_active(
+            self.presentLoggedErrorsCheckButton,
+            script_settings.presentLoggedErrors)
+
+        return vbox
+
+    def setAppPreferences(self, prefs):
+        """Write out the application specific preferences lines and set the
+        new values.
+
+        Arguments:
+        - prefs: file handle for application preferences.
+        """
+
+        prefs.writelines("\n")
+        script_settings.presentLoggedErrors = \
+                 self.presentLoggedErrorsCheckButton.get_active()
+        prefs.writelines("%s.presentLoggedErrors = %s\n" % \
+                         ("orca.scripts.apps.packagemanager.script_settings",
+                          script_settings.presentLoggedErrors))
+
     def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
         """Called when the visual object with focus changes.
 
@@ -94,11 +150,11 @@ class Script(default.Script):
 
         # Prevent chattiness when arrowing out of or into a link.
         #
-        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent) \
-           and orca_state.lastNonModifierKeyEvent.event_string in \
-               ["Left", "Right", "Up", "Down"] \
+        lastKey, mods = self.utilities.lastKeyAndModifiers()
+        if lastKey in ["Left", "Right", "Up", "Down"] \
            and event and event.type.startswith("focus:") \
-           and (self.isLink(oldLocusOfFocus) or self.isLink(newLocusOfFocus)):
+           and (self.utilities.isLink(oldLocusOfFocus) \
+           or self.utilities.isLink(newLocusOfFocus)):
             orca.setLocusOfFocus(event, newLocusOfFocus, False)
             return
 
@@ -124,9 +180,8 @@ class Script(default.Script):
         # script doesn't ignore this event.
         #
         if event.source != orca_state.locusOfFocus \
-           and self.getAncestor(event.source,
-                                [pyatspi.ROLE_HTML_CONTAINER],
-                                [pyatspi.ROLE_FRAME]):
+           and self.utilities.ancestorWithRole(
+            event.source, [pyatspi.ROLE_HTML_CONTAINER], [pyatspi.ROLE_FRAME]):
             orca.setLocusOfFocus(event, event.source, False)
 
         default.Script.onCaretMoved(self, event)
@@ -155,9 +210,6 @@ class Script(default.Script):
         - event: the Event
         """
 
-        # TODO - JD: This might be a good candidate for a braille "flash
-        # message."
-        #
         if event.source.getRole() == pyatspi.ROLE_FRAME \
            and event.type.startswith("object:state-changed:busy"):
             # The busy cursor gets set/unset frequently. It's only worth
@@ -171,52 +223,83 @@ class Script(default.Script):
                 # Translators: this is in reference to loading a web page
                 # or some other content.
                 #
-                speech.speak(_("Loading.  Please wait."))
+                msg = _("Loading.  Please wait.")
+                speech.speak(msg)
+                self.displayBrailleMessage(
+                    msg, flashTime=settings.brailleFlashTime)
                 self._isBusy = True
             elif event.detail1 == 0 and self._isBusy:
                 # Translators: this is in reference to loading a web page
                 # or some other content.
                 #
-                speech.speak(_("Finished loading."))
+                msg = _("Finished loading.")
+                speech.speak(msg)
+                self.displayBrailleMessage(
+                    msg, flashTime=settings.brailleFlashTime)
                 self._isBusy = False
             return
 
+        if script_settings.presentLoggedErrors \
+           and not self._presentedStatusBarIcon \
+           and event.source.getRole() == pyatspi.ROLE_PANEL \
+           and event.type.startswith("object:state-changed:showing") \
+           and event.detail1:
+            obj = self.findStatusBarIcon()
+            while obj and not self.utilities.isSameObject(obj, event.source):
+                obj = obj.parent
+            if obj:
+                # Translators: The Package Manager application notifies the
+                # user of minor errors by displaying an icon in the status
+                # bar and adding them to an error log rather than displaying
+                # the error in a dialog box. This is the message Orca will
+                # present to inform the user that this has occurred.
+                #
+                msg = _("An error occurred. View the error log for details.")
+                speech.speak(msg)
+                self.displayBrailleMessage(
+                    msg, flashTime=settings.brailleFlashTime)
+                self._presentedStatusBarIcon = True
+
         default.Script.onStateChanged(self, event)
 
-    def findStatusBar(self, obj):
-        """Returns the status bar in the window which contains obj.
-        Overridden here because Packagemanager seems to have multiple
-        status bars which claim to be SHOWING and VISIBLE. The one we
-        want should be displaying text, whereas the others are not.
+    def onWindowActivated(self, event):
+        """Called whenever a toplevel window is activated.
+
+        Arguments:
+        - event: the Event
         """
 
-        # There are some objects which are not worth descending.
-        #
-        skipRoles = [pyatspi.ROLE_TREE,
-                     pyatspi.ROLE_TREE_TABLE,
-                     pyatspi.ROLE_TABLE]
+        if self._presentedStatusBarIcon and not self.findStatusBarIcon():
+            self._presentedStatusBarIcon = False
 
-        if obj.getState().contains(pyatspi.STATE_MANAGES_DESCENDANTS) \
-           or obj.getRole() in skipRoles:
-            return
+        default.Script.onWindowActivated(self, event)
 
-        statusBar = None
-        for i in range(obj.childCount - 1, -1, -1):
-            if obj[i].getRole() == pyatspi.ROLE_STATUS_BAR:
-                statusBar = obj[i]
-            elif not obj[i] in skipRoles:
-                statusBar = self.findStatusBar(obj[i])
+    def findStatusBarIcon(self, statusBar=None):
+        """Locates the icon which is sometimes found to the left of the
+        packagemanager status bar.
 
-            if statusBar:
-                try:
-                    text = statusBar.queryText()
-                except:
-                    pass
-                else:
-                    if text.characterCount:
-                        break
+        Arguments:
+        - statusBar: packagemanager's status bar
 
-        return statusBar
+        Returns the accessible for the icon if the icon is found and
+        showing.
+        """
+
+        icon = None
+        if not statusBar and self.app.childCount:
+            # Be sure we're looking in PM's main window.
+            #
+            statusBar = self.utilities.statusBar(self.app[0])
+
+        if statusBar:
+            i = statusBar.getIndexInParent()
+            if i > 0:
+                icons = self.utilities.descendantsWithRole(
+                    statusBar.parent[i - 1], pyatspi.ROLE_ICON)
+                if icons:
+                    icon = icons[0]
+
+        return icon
 
     def stopSpeechOnActiveDescendantChanged(self, event):
         """Whether or not speech should be stopped prior to setting the
@@ -284,8 +367,9 @@ class Script(default.Script):
 
         default.Script._presentTextAtNewCaretPosition(self, event, otherObj)
 
-        if not self.isSameObject(event.source, self._lastObjectPresented) \
-           and self.getAncestor(
+        if not self.utilities.isSameObject(
+            event.source, self._lastObjectPresented) \
+           and self.utilities.ancestorWithRole(
             event.source, [pyatspi.ROLE_HTML_CONTAINER], [pyatspi.ROLE_FRAME]):
             self.updateBraille(event.source)
 
@@ -299,7 +383,7 @@ class Script(default.Script):
         - extra: extra Region to add to the end
         """
 
-        if not self.getAncestor(
+        if not self.utilities.ancestorWithRole(
            obj, [pyatspi.ROLE_HTML_CONTAINER], [pyatspi.ROLE_FRAME]):
             return default.Script.updateBraille(self, obj, extraRegion)
 
@@ -308,29 +392,25 @@ class Script(default.Script):
         except:
             return default.Script.updateBraille(self, obj, extraRegion)
 
-        braille.clear()
-        line = braille.Line()
-        braille.addLine(line)
-
+        line = self.getNewBrailleLine(clearBraille=True, addLine=True)
         focusedRegion = None
         contents = self.getLineContentsAtOffset(obj, text.caretOffset)
         for i, content in enumerate(contents):
             child, startOffset, endOffset, string = content
-            isFocusedObj = self.isSameObject(child, obj)
-            regions = [braille.Text(child,
-                                    startOffset=startOffset,
-                                    endOffset=endOffset)]
-
+            isFocusedObj = self.utilities.isSameObject(child, obj)
+            regions = [self.getNewBrailleText(child,
+                                              startOffset=startOffset,
+                                              endOffset=endOffset)]
             if isFocusedObj:
                 focusedRegion = regions[0]
 
-            line.addRegions(regions)
+            self.addBrailleRegionsToLine(regions, line)
 
         if extraRegion:
-            line.addRegion(extraRegion)
+            self.addBrailleRegionToLine(extraRegion, line)
 
-        braille.setFocus(focusedRegion, getLinkMask=False)
-        braille.refresh(panToCursor=True, getLinkMask=False)
+        self.setBrailleFocus(focusedRegion, getLinkMask=False)
+        self.refreshBraille(panToCursor=True, getLinkMask=False)
 
     def sayCharacter(self, obj):
         """Speak the character at the caret. Overridden here because the
@@ -342,7 +422,7 @@ class Script(default.Script):
                interface
         """
 
-        if not self.getAncestor(
+        if not self.utilities.ancestorWithRole(
            obj, [pyatspi.ROLE_HTML_CONTAINER], [pyatspi.ROLE_FRAME]):
             return default.Script.sayCharacter(self, obj)
 
@@ -351,10 +431,8 @@ class Script(default.Script):
         except:
             return
 
-        if text.caretOffset == text.characterCount \
-           and isinstance(orca_state.lastInputEvent,
-                          input_event.KeyboardEvent) \
-           and orca_state.lastNonModifierKeyEvent.event_string == "Right":
+        lastKey, mods = self.utilities.lastKeyAndModifiers()
+        if text.caretOffset == text.characterCount and lastKey == "Right":
             nextObj = self.getRelationTarget(obj, pyatspi.RELATION_FLOWS_TO)
             if nextObj and nextObj.getRole() == pyatspi.ROLE_TEXT:
                 obj = nextObj
@@ -364,7 +442,7 @@ class Script(default.Script):
     def sayLine(self, obj):
         """Speaks the line at the current caret position."""
 
-        if not self.getAncestor(
+        if not self.utilities.ancestorWithRole(
            obj, [pyatspi.ROLE_HTML_CONTAINER], [pyatspi.ROLE_FRAME]):
             return default.Script.sayLine(self, obj)
 
@@ -379,7 +457,7 @@ class Script(default.Script):
             if len(contents) > 1 and not line.strip():
                 continue
 
-            isLink = self.isLink(child)
+            isLink = self.utilities.isLink(child)
 
             if len(line) and line != "\n":
                 if line.decode("UTF-8").isupper():
@@ -389,7 +467,7 @@ class Script(default.Script):
                 else:
                     voice = self.voices[settings.DEFAULT_VOICE]
 
-                line = self.adjustForRepeats(line)
+                line = self.utilities.adjustForRepeats(line)
                 speech.speak(line, voice)
                 if isLink:
                     speech.speak(self.speechGenerator.getRoleName(
@@ -522,45 +600,6 @@ class Script(default.Script):
         verticalCenter2 = extents2[1] + extents2[3] / 2
         return abs(verticalCenter1 - verticalCenter2) <= 5
 
-    def isLink(self, obj):
-        """Returns True if this is a text object serving as a link.
-
-        Arguments:
-        - obj: an accessible
-        """
-
-        if not obj:
-            return False
-
-        # Images seem to be exposed as ROLE_PANEL and implement very few of
-        # the accessibility interfaces.
-        #
-        if obj.getRole() == pyatspi.ROLE_PANEL and not obj.childCount \
-           and obj.getState().contains(pyatspi.STATE_FOCUSABLE) \
-           and self.getAncestor(obj,
-                                [pyatspi.ROLE_HTML_CONTAINER],
-                                [pyatspi.ROLE_FRAME]):
-            return True
-
-        try:
-            text = obj.queryText()
-        except:
-            return False
-        else:
-            return self.getLinkIndex(obj, text.caretOffset) >= 0
-
-    def isTextArea(self, obj):
-        """Returns True if obj is a GUI component that is for entering text.
-
-        Arguments:
-        - obj: an accessible
-        """
-
-        if self.isLink(obj):
-            return False
-
-        return default.Script.isTextArea(self, obj)
-
     def isSearchEntry(self, obj):
         """Attempts to distinguish the Search entry from other accessibles.
 
@@ -574,7 +613,7 @@ class Script(default.Script):
         # should change, we'll need to make our criteria more specific.
         #
         if obj and obj.getRole() == pyatspi.ROLE_TEXT \
-           and self.getAncestor( \
+           and self.utilities.ancestorWithRole(
             obj, [pyatspi.ROLE_TOOL_BAR], [pyatspi.ROLE_FRAME]):
             return True
 
@@ -607,10 +646,10 @@ class Script(default.Script):
                         except:
                             col = -1
                         else:
-                            index = self.getCellIndex(obj)
+                            index = self.utilities.cellIndex(obj)
                             col = table.getColumnAtIndex(index)
                         if col == 0:
-                            top = self.getTopLevel(obj)
+                            top = self.utilities.topLevelObject(obj)
                             if top and top.getRole() == pyatspi.ROLE_FRAME:
                                 return True
                         return False
