@@ -62,9 +62,9 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         if handleRibbonButton:
             result.append(_("Display more options"))
         elif handleTabButton:
-            result.append(self._script.getDisplayedText(obj.parent[1]))
+            result.append(self._script.utilities.displayedText(obj.parent[1]))
         else:
-            result.append(
+            result.extend(
                 speech_generator.SpeechGenerator._generateLabelAndName(
                     self, obj, **args))
 
